import cv2
import math
import numpy as np

MARGIN = 8


def get_contour_feature(bimg):
    contours, hierarchy = cv2.findContours(bimg, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
    s, l = 0, 0
    x0, y0, x1, y1 = bimg.shape[1], bimg.shape[0], 0, 0

    for cnt in contours:
        if len(cnt) > 0:
            s += cv2.contourArea(cnt)
            l += cv2.arcLength(cnt, True)

            x, y, w, h = cv2.boundingRect(cnt)
            x0 = min(x0, x)
            y0 = min(y0, y)
            x1 = max(x1, x + w)
            y1 = max(y1, y + h)

    circleness = 2 * math.sqrt(math.pi * s) / l
    x0 = max(0, x0 - MARGIN)
    y0 = max(0, y0 - MARGIN)
    x1 = min(bimg.shape[1], x1 + MARGIN)
    y1 = min(bimg.shape[0], y1 + MARGIN)

    return circleness, x0, y0, x1, y1

def get_feature_from_blob(path):
    weight = np.array([1,1,1,1,1,1,1,1])

    if path is None:
        return np.random.rand(8) * weight
    else:
        feature = np.zeros(8)
        img = cv2.imread(path)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        _, bimg = cv2.threshold(img, 128, 255, cv2.THRESH_BINARY)
        feature[0], x0, y0, x1, y1 = get_contour_feature(bimg)
        kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE,(11,11))
        bimg = cv2.morphologyEx(bimg, cv2.MORPH_CLOSE, kernel)
        feature[1], _, _, _, _ = get_contour_feature(bimg)

        dw = (x1 - x0) - (y1 - y0)
        if dw > 0:
            y0 -= dw // 2
            y1 += dw // 2
        else:
            x0 += dw // 2
            x1 -= dw // 2
        feature[2] = np.mean(bimg[y0:y1, x0:(x0*2+x1)//3]) / 255
        feature[3] = np.mean(bimg[y0:y1, (x0*2+x1)//3:(x0+x1)*2//3]) / 255
        feature[4] = np.mean(bimg[y0:y1, (x0+x1*2)//3:x1]) / 255
        feature[5] = np.mean(bimg[y0:(y0*2+y1)//3, x0:x1]) / 255
        feature[6] = np.mean(bimg[(y0*2+y1)//3:(y0+y1)*2//3, x0:x1]) / 255
        feature[7] = np.mean(bimg[(y0+y1*2)//3:y1, x0:x1]) / 255
        return feature * weight
