import torchvision
import torch
from torch import nn
from torchvision import transforms
from PIL import Image
import numpy as np

device = None
model = None
transform = None

class ResNetBackBone(nn.Module):
    def __init__(self, resnet_type="resnet18", pretrained=True):
        super().__init__()
        print(resnet_type)
        if resnet_type == "resnet18":
            self.model = torchvision.models.resnet18(pretrained=pretrained)

    def forward(self, x):
        x = self.model.conv1(x)
        x = self.model.bn1(x)
        x = self.model.relu(x)
        x = self.model.maxpool(x)

        x = self.model.layer1(x)
        x = self.model.layer2(x)
        x = self.model.layer3(x)
        x = self.model.layer4(x)

        x = self.model.avgpool(x)
        x = torch.flatten(x, 1)
        #x = self.model.fc(x)

        return x

def get_device(use_gpu):
    if use_gpu and torch.cuda.is_available():
        torch.backends.cudnn.deterministic = True
        return torch.device("cuda")
    else:
        return torch.device("cpu")

def get_transform():
    return transforms.Compose(
        [
            transforms.Resize(256),
            transforms.CenterCrop(224),
            transforms.ToTensor(),
            transforms.Normalize(
                mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]
            ),
        ]
    )

def get_feature_from_cnn(path):
    global device
    global model
    global transform

    if path is not None:
        if device is None:
            device = get_device(use_gpu=True)
            model = ResNetBackBone().to(device)
            model.eval()
            transform = get_transform()

        img = Image.open(path)
        inputs = transform(img)
        inputs = inputs.unsqueeze(0).to(device)
        outputs = model(inputs)
        outputs = outputs[0].to('cpu').detach().numpy()
        return outputs

    else:
        return np.random.rand(512)
